require_relative "gitlab_base"
require "bigdecimal/util"

class Chef
  class Resource
    class GitlabApplicationSettings < Chef::Resource::GitlabBase
      resource_name :gitlab_application_settings

      property :admin_notification_email, String
      property :after_sign_out_path, String
      property :after_sign_up_text, String
      property :akismet_api_key, String
      property :akismet_enabled, [TrueClass, FalseClass]
      property :allow_group_owners_to_manage_ldap, [TrueClass, FalseClass]
      property :allow_local_requests_from_hooks_and_services, [TrueClass, FalseClass]
      property :authorized_keys_enabled, [TrueClass, FalseClass]
      property :auto_devops_domain, String
      property :auto_devops_enabled, [TrueClass, FalseClass]
      property :check_namespace_plan, [TrueClass, FalseClass]
      property :clientside_sentry_dsn, String
      property :clientside_sentry_enabled, [TrueClass, FalseClass]
      property :container_registry_token_expire_delay, Integer
      property :default_artifacts_expire_in, String
      property :default_branch_protection, Integer
      property :default_group_visibility, String
      property :default_project_visibility, String
      property :default_projects_limit, Integer
      property :default_snippet_visibility, String
      property :disabled_oauth_sign_in_sources, Array
      property :domain_blacklist, Array
      property :domain_blacklist_enabled, [TrueClass, FalseClass]
      property :domain_whitelist, Array
      property :dsa_key_restriction, Integer
      property :ecdsa_key_restriction, Integer
      property :ed25519_key_restriction, Integer
      property :elasticsearch_aws, [TrueClass, FalseClass]
      property :elasticsearch_aws_access_key, String
      property :elasticsearch_aws_region, String
      property :elasticsearch_aws_secret_access_key, String
      property :elasticsearch_experimental_indexer, [TrueClass, FalseClass]
      property :elasticsearch_indexing, [TrueClass, FalseClass]
      property :elasticsearch_search, [TrueClass, FalseClass]
      property :elasticsearch_url, [String, Array]
      property :elasticsearch_limit_indexing, [TrueClass, FalseClass]
      property :elasticsearch_project_ids, Array
      property :elasticsearch_namespace_ids, Array
      property :email_additional_text, String
      property :email_author_in_body, [TrueClass, FalseClass]
      property :enabled_git_access_protocol, String
      property :enforce_terms, [TrueClass, FalseClass]
      property :external_auth_client_cert, String
      property :external_auth_client_key, String
      property :external_auth_client_key_pass, String
      property :external_authorization_service_enabled, [TrueClass, FalseClass]
      property :external_authorization_service_default_label, String
      property :external_authorization_service_timeout, Float
      property :external_authorization_service_url, String
      property :file_template_project_id, Integer
      property :first_day_of_week, Integer
      property :geo_status_timeout, Integer
      property :gitaly_timeout_default, Integer
      property :gitaly_timeout_fast, Integer
      property :gitaly_timeout_medium, Integer
      property :gravatar_enabled, [TrueClass, FalseClass]
      property :hashed_storage_enabled, [TrueClass, FalseClass]
      property :help_page_hide_commercial_content, [TrueClass, FalseClass]
      property :help_page_support_url, String
      property :help_page_text, String
      property :help_text, String
      property :hide_third_party_offers, [TrueClass, FalseClass]
      property :home_page_url, String
      property :housekeeping_bitmaps_enabled, [TrueClass, FalseClass]
      property :housekeeping_enabled, [TrueClass, FalseClass]
      property :housekeeping_full_repack_period, Integer
      property :housekeeping_gc_period, Integer
      property :housekeeping_incremental_repack_period, Integer
      property :html_emails_enabled, [TrueClass, FalseClass]
      property :instance_statistics_visibility_private, [TrueClass, FalseClass]
      property :import_sources, Array
      property :max_artifacts_size, Integer
      property :max_attachment_size, Integer
      property :max_pages_size, Integer
      property :metrics_enabled, [TrueClass, FalseClass]
      property :metrics_host, String
      property :metrics_method_call_threshold, Integer
      property :metrics_packet_size, Integer
      property :metrics_pool_size, Integer
      property :metrics_port, Integer
      property :metrics_sample_interval, Integer
      property :metrics_timeout, Integer
      property :mirror_available, [TrueClass, FalseClass]
      property :mirror_capacity_threshold, Integer
      property :mirror_max_capacity, Integer
      property :mirror_max_delay, Integer
      property :pages_domain_verification_enabled, [TrueClass, FalseClass]
      property :password_authentication_enabled_for_git, [TrueClass, FalseClass]
      property :password_authentication_enabled_for_web, [TrueClass, FalseClass]
      property :performance_bar_allowed_group_id, String
      property :performance_bar_allowed_group_path, String
      property :performance_bar_enabled, [TrueClass, FalseClass]
      property :plantuml_enabled, [TrueClass, FalseClass]
      property :plantuml_url, String
      property :polling_interval_multiplier, BigDecimal, coerce: proc { |x| BigDecimal(x) }
      property :project_export_enabled, [TrueClass, FalseClass]
      property :prometheus_metrics_enabled, [TrueClass, FalseClass]
      property :pseudonymizer_enabled, [TrueClass, FalseClass]
      property :recaptcha_enabled, [TrueClass, FalseClass]
      property :recaptcha_private_key, String
      property :recaptcha_site_key, String
      property :repository_checks_enabled, [TrueClass, FalseClass]
      property :repository_size_limit, Integer
      property :repository_storages, Array
      property :require_two_factor_authentication, [TrueClass, FalseClass]
      property :restricted_visibility_levels, Array
      property :rsa_key_restriction, Integer
      property :send_user_confirmation_email, [TrueClass, FalseClass]
      property :sentry_dsn, String
      property :sentry_enabled, [TrueClass, FalseClass]
      property :session_expire_delay, Integer
      property :shared_runners_enabled, [TrueClass, FalseClass]
      property :shared_runners_minutes, Integer
      property :shared_runners_text, String
      property :sign_in_text, String
      property :signin_enabled, [TrueClass, FalseClass]
      property :signup_enabled, [TrueClass, FalseClass]
      property :slack_app_enabled, [TrueClass, FalseClass]
      property :slack_app_id, String
      property :slack_app_secret, String
      property :slack_app_verification_token, String
      property :terminal_max_session_time, Integer
      property :terms, String
      property :throttle_authenticated_api_enabled, [TrueClass, FalseClass]
      property :throttle_authenticated_api_period_in_seconds, Integer
      property :throttle_authenticated_api_requests_per_period, Integer
      property :throttle_authenticated_web_enabled, [TrueClass, FalseClass]
      property :throttle_authenticated_web_period_in_seconds, Integer
      property :throttle_authenticated_web_requests_per_period, Integer
      property :throttle_unauthenticated_enabled, [TrueClass, FalseClass]
      property :throttle_unauthenticated_period_in_seconds, Integer
      property :throttle_unauthenticated_requests_per_period, Integer
      property :two_factor_grace_period, Integer
      property :unique_ips_limit_enabled, [TrueClass, FalseClass]
      property :unique_ips_limit_per_user, Integer
      property :unique_ips_limit_time_window, Integer
      property :usage_ping_enabled, [TrueClass, FalseClass]
      property :user_default_external, [TrueClass, FalseClass]
      property :user_oauth_applications, [TrueClass, FalseClass]
      property :user_show_add_ssh_key_message, [TrueClass, FalseClass]
      property :version_check_enabled, [TrueClass, FalseClass]
      property :local_markdown_version, Integer
      property :geo_node_allowed_ips, String

      load_current_value do |new_resource|
        begin
          settings = gitlab_client.application_settings
        rescue Gitlab::Error::ResponseError => e
          Chef::Log.error("Gitlab responded with an error (#{e.class}): #{e.message}")
          raise
        end

        settings.to_h.each do |k, v|
          next if v.nil?
          setting = k.to_sym

          if api_to_property_map.key?(setting)
            set_property_from_api_response(setting, v)
            next
          end

          if respond_to?(setting)
            send(setting, v)
          else
            Chef::Log.debug("Gitlab responded with an additional setting (#{setting}) that isn't supported at this time. Submit a bug request if you need to set this.")
          end
        end
      end

      action :configure do
        converge_if_changed do
          begin
            new_resource.gitlab_client.edit_application_settings(gitlab_edit_application_settings_options)
          rescue Gitlab::Error::ResponseError => e
            Chef::Log.error("Gitlab responded with an error (#{e.class}): #{e.message}")
            raise
          end
        end
      end

      action_class do
        def gitlab_edit_application_settings_options
          gitlab_resource_options
        end
      end

      private

      def set_property_from_api_response(property, value)
        converter = api_to_property_map[property.to_sym][:proc]
        resource_property = api_to_property_map[property.to_sym][:property]
        send(resource_property, converter.call(value))
      end

      def api_to_property_map
        {
          domain_blacklist_raw: {
            property: :domain_blacklist,
            proc: Proc.new { |v| v.split(",") },
          },
          domain_whitelist_raw: {
            property: :domain_whitelist,
            proc: Proc.new { |v| v.split(",") },
          },
        }
      end
    end
  end
end
