require "gitlab"

class Chef
  class Resource
    class GitlabBase < Chef::Resource
      resource_name :gitlab_resource_abstract_base_class # this prevents magickal class name DSL wiring

      property :endpoint, String, required: true, desired_state: false
      property :private_token, String, required: true, desired_state: false, sensitive: true

      # Private API
      property :id, Integer, desired_state: false

      def gitlab_client
        @gitlab_client ||= Gitlab::Client.new(endpoint: endpoint, private_token: private_token)
      end

      action_class do
        def gitlab_resource_properties
          new_resource.class.state_properties.map { |property| property.name }
        end

        def gitlab_resource_options
          gitlab_resource_properties.inject({}) do |memo, property|
            value = new_resource.send(property)
            memo[property] = value unless value.nil?
            memo
          end
        end
      end
    end
  end
end
