#
# Cookbook:: gitlab_resources
# Spec:: gitlab_application_settings
#
# The MIT License (MIT)
#
# Copyright:: 2019, Jason Barnett
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

require "spec_helper"

GITLAB_ENDPOINT = "https://gitlab.example.com/api/v4".freeze
GITLAB_PRIVATE_TOKEN = "yuY_vkuS3gFKpWhsJNuY".freeze

describe "gitlab_application_settings" do
  platform "centos", "7"

  recipe do
    gitlab_application_settings 'gitlab ee settings' do
      endpoint GITLAB_ENDPOINT
      private_token GITLAB_PRIVATE_TOKEN

      auto_devops_enabled false
      restricted_visibility_levels ["public"]
      password_authentication_enabled_for_web false
      password_authentication_enabled_for_git false
      signup_enabled false
      user_show_add_ssh_key_message true
      default_artifacts_expire_in "7 days"
      default_branch_protection 2
      default_group_visibility "private"
      default_project_visibility "private"
      default_snippet_visibility "private"
      first_day_of_week 1
      gravatar_enabled false
      html_emails_enabled true
      max_artifacts_size 350
    end
  end

  context "stepping into resource" do
    step_into :gitlab_application_settings

    before do
      allow_any_instance_of(Gitlab::Client).to receive(:application_settings).and_return({})
      allow_any_instance_of(Gitlab::Client).to receive(:edit_application_settings).with(anything).and_return(nil)
    end

    it "creates Gitlab::Client instance configured with endpoint and private token" do
      expect(Gitlab::Client).to receive(:new).with(
        endpoint: GITLAB_ENDPOINT,
        private_token: GITLAB_PRIVATE_TOKEN
      ).at_most(:twice).and_call_original

      chef_run
    end
  end

  context "in a recipe" do
    it do
      is_expected.to configure_gitlab_application_settings("gitlab ee settings").with(
        auto_devops_enabled: false,
        restricted_visibility_levels: ["public"],
        password_authentication_enabled_for_web: false,
        password_authentication_enabled_for_git: false,
        signup_enabled: false,
        user_show_add_ssh_key_message: true,
        default_artifacts_expire_in: "7 days",
        default_branch_protection: 2,
        default_group_visibility: "private",
        default_project_visibility: "private",
        default_snippet_visibility: "private",
        first_day_of_week: 1,
        gravatar_enabled: false,
        html_emails_enabled: true,
        max_artifacts_size: 350
      )
    end

    it "converges successfully" do
      expect { chef_run }.to_not raise_error
    end
  end
end
